#pragma once

#include <X-GSD/Component.hpp>
#include <X-GSD/Entity.hpp> // Completes forward declaration in Component
#include <X-GSD/ComponentCollider.hpp>
#include <X-GSD/PhysicState.hpp>
#include <X-GSD/Integrator.hpp>

#include <SFML/Graphics/Transformable.hpp>

namespace xgsd {
    
    /*
     ComponentRigidBody class. Add this component to an entity to give it physics behavior (the entity will
     get its position updated according to physics). If the Entity also holds a Collider component, it will
     trigger collisions. If the RigidBody is set as Kinematic, the Entity will not get its position updated
     automatically, but it will still trigger collisions.
     
     Attention: This class is in very basic state and functionality, totally subject to change.
     */
    class ComponentRigidBody : public Component
    {
        // Methods
    public:
        ComponentRigidBody(bool isKinematic);
        ~ComponentRigidBody();
            
        void                onEntityAttach() override;
        void                update(const HiResDuration& dt) override;
        
        void                returnToLastPhysicsState();
        // TODO: override collisionHandler method for a physics automatic collision response (modify the physics state accordingly to that collision)
        
        // TODO: Add setters and getters regarding the mPhysics and use them at JSON loading
        PhysicState&        getPhysicsState();
        
        void                pausePhysics();
        void                resumePhysics();
        
        // Variables (member / properties)
    private:
        
        bool                mKinematic; // If true, the Entity does not get affected by physics, but still triggers collisions
        bool                mPausedPhysics;
        
        PhysicState         mPhysics;
        PhysicState         mLastPhysicsState;
        sf::Transformable   mLastTransformable;
        
        // TODO: More member variables like "affectedByGravity", "mass", "force", "friction", "airFriction", "bounceRatio"...
    };
} // namespace xgsd
