#pragma once

#include <X-GSD/Time.hpp>
#include <X-GSD/PhysicState.hpp>

#include <SFML/System/Vector2.hpp>
#include <SFML/Graphics/Transformable.hpp>


namespace xgsd {
    
    // Needed struct for RK4 method.
    struct Derivative
    {
        sf::Vector2f dp; // dp/dt = velocity
        sf::Vector2f dv; // dv/dt = acceleration
    };
    
    
    // Integrator class. Implements Euler and RK4 integration methods.
    class Integrator
    {
        // Class methods
    public:
        void static             Euler(PhysicState& physics,
                                      PhysicState& lastPhysics,
                                      sf::Transformable& transf,
                                      sf::Transformable& lastTransf,
                                      const HiResDuration& dt);
        
        void static             RK4(PhysicState& physics,
                                    PhysicState& lastPhysics,
                                    sf::Transformable& transf,
                                    sf::Transformable& lastTransf,
                                    const HiResDuration& dt);
        
    private:
        Derivative static       evaluateRK4(const PhysicState& initialPhysics,
                                            const sf::Transformable& initialTransf,
                                            const HiResDuration& dt,
                                            const Derivative& d);
        
        sf::Vector2f static     accelerationRK4(const PhysicState&
                                                physics);
    };
    
} // namespace xgsd
