#include <X-GSD/PhysicsEngine.hpp>

using namespace xgsd;

// Very basic collision detection algorithm based on axis-aligned bounding boxes intersection
// TODO: onEnter/onExit collision methods? Store list of already colliding entities?
void PhysicsEngine::checkCollisions() {
    
    // Check between dynamic colliders first (entities which have a collider and a rigidBody)
    for (auto iterD = dynamicColliders.begin(); iterD != dynamicColliders.end(); ++iterD) {
        
        // If the entity containing this collider is pending of destruction, skip it
        if ((*iterD)->entity->isDestroyPending())
            continue;
        
        sf::FloatRect intersection;
        
        // Get the global bounds rect of one entity
        sf::FloatRect rectD = (*iterD)->entity->getWorldTransform().transformRect((*iterD)->getRectBounds());
        
        // Check between dynamic colliders (entities which have a collider and a rigidBody)
        for (auto iterD2 = std::next(iterD); iterD2 != dynamicColliders.end(); ++iterD2) {
            
            /* The starting point of the iterator is the next item of iterD because dynamic colliders
             must check collision between them avoiding repetition (1-2 is the same as 2-1). Also,
             it avoids self-collision detecton (1-1, 2-2, etc).
             */
            
            // If the entity containing this collider is pending of destruction, skip it
            if ((*iterD2)->entity->isDestroyPending() || (*iterD)->entity->isDestroyPending())
                continue;
            
            // Get the global bounds rect of the other entity
            sf::FloatRect rectD2 = (*iterD2)->entity->getWorldTransform().transformRect((*iterD2)->getRectBounds());
            
            // Check collision with SFML and call collisionHandler of both entities
            if(rectD.intersects(rectD2, intersection)){
                (*iterD)->entity->collisionHandler((*iterD2)->entity, intersection);
                (*iterD2)->entity->collisionHandler((*iterD)->entity, intersection);
            }
        }
        
        // Check between dynamic (entities which have a collider and a rigidBody) and static colliders (entities which have a collider but no rigidBody)
        for (auto iterS = staticColliders.begin(); iterS != staticColliders.end(); ++iterS) {
            
            // If the entity containing this collider is pending of destruction, skip it
            if ((*iterS)->entity->isDestroyPending() || (*iterD)->entity->isDestroyPending())
                continue;
            
            // Get the global bounds rect of the other entity
            sf::FloatRect rectS = (*iterS)->entity->getWorldTransform().transformRect((*iterS)->getRectBounds());
            
            // Check collision with SFML and call collisionHandler of both entities
            if(rectD.intersects(rectS, intersection)){
                (*iterD)->entity->collisionHandler((*iterS)->entity, intersection);
                (*iterS)->entity->collisionHandler((*iterD)->entity, intersection);
            }
        }
    }
}

void PhysicsEngine::addStaticCollider(xgsd::ComponentCollider *collider)
{
    staticColliders.insert(collider);
}

void PhysicsEngine::addDynamicCollider(xgsd::ComponentCollider *collider)
{
    dynamicColliders.insert(collider);
}

void PhysicsEngine::deleteStaticCollider(xgsd::ComponentCollider *collider)
{
    staticColliders.erase(collider);
}

void PhysicsEngine::deleteDynamicCollider(xgsd::ComponentCollider *collider)
{
    dynamicColliders.erase(collider);
}
